@@ -18,20 +18,27 @@ pub enum Tok<'input> {
     Skip,
 }
 
+enum LexerContext {
+    None,
+    SectionDefinition,
+    InsideSection,
+    InsideString,
+}
+
 pub struct Lexer<'input> {
     chars: std::iter::Peekable<CharIndices<'input>>,
     input: &'input str,
-    inside_str: bool,
-    tab_count: u32,
+    context: LexerContext,
 }
 
+const END_PREFIX: &str = "End";
+
 impl<'input> Lexer<'input> {
     pub fn new(input: &'input str) -> Self {
         Lexer {
             chars: input.char_indices().peekable(),
             input,
-            inside_str: false,
-            tab_count: 0,
+            context: LexerContext::None,
         }
     }
 
@@ -48,18 +55,28 @@ impl<'input> Lexer<'input> {
                         break;
                     }
                     _ => {
-                        if &self.input[i..i + 3] == "End" {
+                        if Lexer::is_close_element(&self.input[i..]) {
+                            self.context = LexerContext::None;
                             return (Tok::CloseElement(&self.input[i..*j]), *j);
-                        };
+                        }
                         return (Tok::Id(&self.input[i..*j]), *j);
                     }
                 },
                 None => {
+                    if Lexer::is_close_element(&self.input[i..]) {
+                        return (Tok::CloseElement(&self.input[i..]), self.input.len());
+                    }
                     return (Tok::Id(&self.input[i..]), self.input.len());
                 }
             }
         }
+        // Skip (
         self.chars.next();
+
+        let section_suffix = "Section";
+        if &self.input[(finish - section_suffix.len())..finish] == section_suffix {
+            self.context = LexerContext::SectionDefinition;
+        };
         (Tok::OpenElement(&self.input[i..finish]), finish)
     }
 
@@ -80,10 +97,12 @@ impl<'input> Lexer<'input> {
     }
 
     fn guid(&mut self, i: usize) -> Option<Spanned<Tok<'input>, usize, ()>> {
+        let finish;
         loop {
             match self.chars.peek() {
                 Some((j, '}')) => {
-                    return Some(Ok((i, Tok::Guid(&self.input[i..*j + 1]), *j)));
+                    finish = *j + 1;
+                    break;
                 }
                 None => {
                     return Some(Ok((i, Tok::Guid(&self.input[i..]), self.input.len())));
@@ -93,6 +112,9 @@ impl<'input> Lexer<'input> {
                 }
             }
         }
+        // Skip {
+        self.chars.next();
+        Some(Ok((i, Tok::Guid(&self.input[i..finish]), finish)))
     }
 
     fn digits_with_dots(&mut self, i: usize) -> Option<Spanned<Tok<'input>, usize, ()>> {
@@ -116,13 +138,16 @@ impl<'input> Lexer<'input> {
     }
 
     fn string(&mut self, i: usize) -> Option<Spanned<Tok<'input>, usize, ()>> {
-        if self.inside_str {
-            // Skip trailing
-            self.inside_str = false;
-            return Some(Ok((i, Tok::Skip, i + 1)));
-        } else {
-            self.inside_str = true;
+        match self.context {
+            LexerContext::InsideString => {
+                self.context = LexerContext::None;
+                return Some(Ok((i, Tok::Skip, i + 1)));
+            }
+            _ => {
+                self.context = LexerContext::InsideString;
+            }
         }
+
         let mut guid = false;
         loop {
             match self.chars.peek() {
@@ -140,9 +165,7 @@ impl<'input> Lexer<'input> {
                         Some(Ok((start, Tok::Str(&self.input[start..finish]), finish)))
                     };
                 }
-                None => {
-                    return Some(Ok((i, Tok::Str(&self.input[i..]), self.input.len())));
-                }
+                None => return Some(Ok((i, Tok::Str(&self.input[i..]), self.input.len()))),
                 _ => {
                     self.chars.next();
                 }
@@ -151,30 +174,43 @@ impl<'input> Lexer<'input> {
     }
 
     fn section_key(&mut self, i: usize) -> Option<Spanned<Tok<'input>, usize, ()>> {
-        self.tab_count += 1;
+        let mut start = i;
+
+        while let Some((j, '\r')) | Some((j, '\n')) | Some((j, '\t')) | Some((j, ' ')) =
+            self.chars.peek()
+        {
+            start = *j + 1;
+            self.chars.next();
+        }
+
+        if Lexer::is_close_element(&self.input[start..]) {
+            self.context = LexerContext::None;
+            return Some(Ok((start, Tok::Skip, start)));
+        }
+
+        match self.context {
+            LexerContext::InsideSection => {}
+            LexerContext::SectionDefinition => self.context = LexerContext::InsideSection,
+            _ => return Some(Ok((start, Tok::Skip, start))),
+        }
 
         loop {
-            // Skip first
-            if self.tab_count == 1 {
-                return Some(Ok((i, Tok::Skip, i + 1)));
-            }
-            let start = i + 1;
             match self.chars.peek() {
                 Some((j, '=')) => {
                     let finish = Lexer::trim_end(self.input, *j);
 
-                    return Some(Ok((
-                        start,
-                        Tok::SectionKey(&self.input[start..finish]),
-                        finish,
-                    )));
+                    let val = &self.input[start..finish];
+
+                    return Some(Ok((start, Tok::SectionKey(val), finish)));
                 }
                 None => {
-                    return Some(Ok((
-                        start,
-                        Tok::SectionKey(&self.input[start..]),
-                        self.input.len(),
-                    )));
+                    let val = &self.input[start..];
+
+                    if Lexer::is_close_element(val) {
+                        return Some(Ok((start, Tok::CloseElement(val), self.input.len())));
+                    }
+
+                    return Some(Ok((start, Tok::SectionKey(val), self.input.len())));
                 }
                 _ => {
                     self.chars.next();
@@ -184,34 +220,43 @@ impl<'input> Lexer<'input> {
     }
 
     fn section_value(&mut self, i: usize) -> Option<Spanned<Tok<'input>, usize, ()>> {
-        if self.tab_count <= 1 {
-            Some(Ok((i, Tok::Eq, i + 1)))
-        } else {
-            let start = Lexer::trim_start(self.input, i + 1);
+        match self.context {
+            LexerContext::InsideSection => {
+                // i + 1 skips equal sign (=)
+                let start = Lexer::trim_start(self.input, i + 1);
 
-            loop {
-                match self.chars.peek() {
-                    Some((j, '\r')) | Some((j, '\n')) => {
-                        self.tab_count = 0;
-                        let finish = *j;
-                        return Some(Ok((
-                            start,
-                            Tok::SectionValue(&self.input[start..finish]),
-                            finish,
-                        )));
-                    }
-                    None => {
-                        return Some(Ok((
-                            start,
-                            Tok::SectionValue(&self.input[start..]),
-                            self.input.len(),
-                        )));
-                    }
-                    _ => {
-                        self.chars.next();
+                loop {
+                    match self.chars.peek() {
+                        Some((j, '\r')) | Some((j, '\n')) => {
+                            let finish = *j;
+                            return Some(Ok((
+                                start,
+                                Tok::SectionValue(&self.input[start..finish]),
+                                finish,
+                            )));
+                        }
+                        None => {
+                            return Some(Ok((
+                                start,
+                                Tok::SectionValue(&self.input[start..]),
+                                self.input.len(),
+                            )));
+                        }
+                        _ => {
+                            self.chars.next();
+                        }
                     }
                 }
             }
+            _ => Some(Ok((i, Tok::Eq, i + 1))),
+        }
+    }
+
+    fn is_close_element(val: &str) -> bool {
+        if val.len() < END_PREFIX.len() {
+            false
+        } else {
+            &val[..END_PREFIX.len()] == END_PREFIX
         }
     }
 
@@ -231,7 +276,7 @@ impl<'input> Lexer<'input> {
 
     fn current(&mut self, i: usize, ch: char) -> Option<Spanned<Tok<'input>, usize, ()>> {
         match ch {
-            '\t' => return self.section_key(i),
+            '\r' | '\n' => return self.section_key(i),
             '=' => return self.section_value(i),
             ',' => return Some(Ok((i, Tok::Comma, i + 1))),
             ')' => return Some(Ok((i, Tok::Skip, i + 1))),
@@ -257,8 +302,7 @@ impl<'input> Iterator for Lexer<'input> {
             let (i, c) = self.chars.next()?;
 
             match c {
-                ' ' | '\n' | '\r' | '}' => {
-                    self.tab_count = 0;
+                ' ' | '\t' => {
                     continue;
                 }
                 _ => {}
@@ -295,7 +339,21 @@ mod tests {
 
     #[test]
     fn lex_solution() {
-        let input = r#"
+        let lexer = Lexer::new(REAL_SOLUTION);
+        for tok in lexer {
+            println!("{:#?}", tok);
+        }
+    }
+
+    #[test]
+    fn lex_solution_no_last_line_break() {
+        let lexer = Lexer::new(REAL_SOLUTION.trim_end());
+        for tok in lexer {
+            println!("{:#?}", tok);
+        }
+    }
+
+    const REAL_SOLUTION: &str = r#"
 Microsoft Visual Studio Solution File, Format Version 12.00
 # Visual Studio 15
 VisualStudioVersion = 15.0.26403.0
@@ -441,9 +499,4 @@ Global
 	EndGlobalSection
 EndGlobal
          "#;
-        let lexer = Lexer::new(input);
-        for tok in lexer {
-            println!("{:#?}", tok);
-        }
-    }
 }
